<?php
/**
 * 2018 Cenový automat (www.cenovyautomat.cz)
 * Univerzální PHP skript pro propojení se službou Cenový automat
 * v1.1 (4.6.2018)
 *
 * NOTICE OF LICENSE
 *
 * This software is provided as is, without warranty of any kind.
 * The author shall not be liable for damages of any kind.
 * Use of this software indicates that you agree.
 *
 * Tento software je poskytován tak, jak je, bez záruky jakéhokoli druhu.
 * Autor nenese odpovědnost za škody jakéhokoliv druhu.
 * Používáním tohoto softwaru znamená, že souhlasíte s výše uvedenými autorskými právy.
 *
 * DISCLAIMER
 *
 * Please do not change this text, remove the link or remove all or any part
 * of the creator copyright notice
 * Please also note that although you are allowed to make modificationsf or your
 * own personal use, you may not distribute the original or the modified code
 * without permission.
 *
 * Prosím, neměňte tento text, nemazejte odkazy, neodstraňujte části a nebo celé
 * oznámení těchto autorských práv.
 * Prosím vezměte také na vědomí, že i když máte možnost provádět změny pro vlastní
 * osobní potřebu, nesmíte distribuovat původní nebo upravený kód bez povolení.
 *
 * @author    Cenový automat <info@cenovyautomat.cz>
 * @copyright  2016 Cenový automat (www.cenovyautomat.cz)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *
 *****************************************************************************************
 *
 * Popis funkcí Univerzálního PHP skriptu
 *
 * Tento skript implementuje do vašeho e-shopu jednoduché API, pomocí kterého s ním může
 * Cenový automat on-line komunikovat.
 * API umožňuje provádět dvě klíčové funkce:
 * 1. Cenový automat posílá e-shopu nové požadované cenu produktů
 * 2. Cenový automat načítá z e-shopu nákupní ceny produktů
 *
 * Komunikace je vždy inicializována z aplikace Cenový automat.
 * Komunikace s API probíhá přes HTTP protokol.
 * Data jsou přenášena s pomocí metody POST ve formátu JSON.
 * Citlivá data (nákupní ceny) jsou zabezpečena pomocí silného šifrovaná a celá komunikace
 * pomocí kontrolních součtů (hash), není tak možné odposlechnout citlivá data
 * nebo jakkoliv narušit integritu dat.
 *
 * Univerzální PHP skript obsahuje kód pro:
 * - odesílání a příjem dat,
 * - šifrování a dešifrování dat,
 * - ověření dat a výpočet kontrolních součtů (hash).
 *
 * Neobsahuje kód, který je specifický pro každý e-shop systém:
 * - aktualizace ceny produktu,
 * - načtení nákupní ceny produktu.
 * Tento kód je třeba doplnit pro váš e-shop systém.
 * Další instrukce najdete níže v postupu implementace a v komentářích ke skriptu.
 * V případě potřeby se neváhejte na nás obrátit, rádi vám s implementací pomůžeme.
 *
 *****************************************************************************************
 *
 * Postup implementace
 *
 * 1. Nahrajte tento php soubor do vašeho e-shopu
 *
 * 2. V aplikaci Cenový automat v konfiguraci Export/Import
 * a. Nastavte E-shop systém: Jiný
 * b. Nastavte Export prodejních cen do e-shopu: Univerzální PHP skript
 * c. Nastavte Import nákupních cen z e-shopu: Univerzální PHP skript
 * d. Vygenerujte (nebo zadejte) API klíč
 * e. Vygenerujte (nebo zadejte) Šifrovací klíč
 * f. Zadejte Url adresa API, to je url adresa umístění tohoto PHP skriptu
 *    (např. http://www.vaseshop.cz/adresar-se-skriptem/cenovy-automat-api.php)
 * g. Stiskněte tlačítko Uložit
 *
 * 3. Níže v tomto PHP skriptu zadejte hodnoty pro proměnné $apiKey a $cryptoKey
 * apiKey = API klíč vygenerovaný (zadaný) v Cenový automat v konfiguraci Export/Import
 * cryptoKey = šifrovací klíč vygenerovaný (zadaný) v Cenový automat v konfiguraci Export/Import
 * (API klíč a šifrovací klíč slouží pro zabezpečení komunikace mezi aplikací Cenový automat a vaším e-shopem)
 *
 * 4. Proveďte test spojení mezi Cenový automat a vaším e-shopem (tímto PHP skriptem)
 * V aplikaci Cenový automat v konfiguraci Export/Import stiskněte tlačítko "Test spojení".
 * Pokud jsou správně zadané hodnoty pro Url adresa API, API klíč a šifrovací klíč, měl by
 * být test spojení úspěšný.
 * V opačném případě zkontrolujte kroky v bodě 2. tohoto návodu.
 *
 * 5. Doplňte kód specifický pro váš e-shop
 * Kód doplňte mezi tagy // VÁŠ KÓD - ZAČÁTEK a // VÁŠ KÓD - KONEC na dvou místech:
 * - Akce: Získat nákupní cenu produktu
 * - Akce: Nastavit novou prodejní cenu produktu
 *
 */


// ZAČÁTEK SKRIPTU

// Zadejte hodnoty pro proměnné apiKey a cryptoKey
// Jde o hodnoty nastavené v aplikaci Cenový automat v konfiguraci Export/Import
$apiKey = '';
$cryptoKey = '';

// Inicializace e-shopu
// Je-li potřeba, vložte kód pro inicializaci vašeho e-shopu.
// Např.:
// require(dirname(__FILE__).'/../config/config.inc.php');
// require(dirname(__FILE__).'/../init.php');

// Načte POST input a dékoduje z JSON formátu
$input = fopen('php://input', 'r');
$response = json_decode(stream_get_contents($input), true);
fclose($input);

// Proměnná $response nyní obsahuje tato data zaslaná z Cenový automat:
// $response['content']['apiKey']: API klíč
// $response['content']['id']: ID produktu
// $response['content']['productNo']: produktové číslo produktu (pro identifikaci produktu, pokud nelze identifikovat pomocí ID)
// $response['content']['ean']: ean produktu (pro identifikaci produktu, pokud nelze identifikovat pomocí ID)
// $response['content']['action']: požadovaná akce (test, getWholesalePrice, setPrice)
// $response['content']['price']: nová požadovaná cena produktu (pro nastavení ceny)
// $response['hash']: hash obsahu v $response['content']


// Kontrola apiKey
// Pokud nesouhlasí, konec skriptu
if ($response['content']['apiKey'] != $apiKey){
    http_response_code(401);
    ob_flush();
    die();
}

// Kontrola hash zprávy
// Pokud hash nesouhlasí = šifrovací klíče nesouhlasí, konec skriptu
if ($response['hash'] != hash_hmac('sha512', json_encode($response['content']), $cryptoKey)){
    http_response_code(400);
    ob_flush();
    die();
}


// Je-li zadaná požadovaná akce (action), pokračuj, jinak chyba
if ($response['content']['action'] !='') {

    // Proveď kód dle požadované akce
    switch ($response['content']['action']) {

        // Akce: Test spojení
        case 'test':
            http_response_code(200);
            echo "Test OK";
            break;


        // Akce: Získat nákupní cenu produktu
        case 'getWholesalePrice':
            if (isset($response['content']['id'])) {

                // VÁŠ KÓD - ZAČÁTEK
                // Zde vložte váš kód, který z e-shopu načte nákupní cenu produktu.

                // Nalezení produktu dle ID
                // Proměnná $response['content']['id'] obsahuje ID produktu, pro který je požadována nákupní cena.
                // Např.:
                // $product = new Product($response['content']['id']);

                // Kontrola, zda-li produkt s tímto ID byl nalezen
                // Např.:
                // if ($product->id != ''){  //pokud $product->id není prázné, produkt byl nalezen
                if (true){

                // Nákupní cenu s DPH uložte do proměnné $price (ve formátu float).
                // Cena může být celé nebo desetinné číslo (123 nebo 123.45), ne string s desetinnou čárkou (123,45).
                // Pokud váš e-shop standardně pracuje s cenami bez DPH, napomeňte provést přepočet na cenu s DPH.
                // Např.:
                // $price = $product->wholesale_price;

                // VÁŠ KÓD - KONEC

                // Nákupní cena je zašifrována, příprava odpovědi a odeslání zpět do Cenový automat
                $price = openssl_encrypt($price, 'aes-128-cbc', $cryptoKey);
                $content['content']['price'] = $price;
                $content['content']['apiKey'] = $apiKey;
                $content['hash'] = hash_hmac('sha512', json_encode($content['content']), $cryptoKey);
                $content = json_encode($content);
                http_response_code(200);
                echo $content;

            } else {
                //Chyba. Produkt se zadaným ID nebyl v e-shopu nalezen.
                http_response_code(404);
                echo "Product with given ID not found";
            }
    } else {
        //Chyba. ID nebylo zadáno.
        http_response_code(400);
        echo "Product ID not specified";

    }
    break;


    // Akce: Nastavit novou prodejní cenu produktu
case 'setPrice':
            if (isset($response['content']['id'])) {

                // VÁŠ KÓD - ZAČÁTEK
                // Zde vložte váš kód, který v e-shopu uloží novou cenu produktu.

                // Nalezení produktu dle ID, productNo nebo EAN
                // Proměnná $response['content']['id'] obsahuje ID produktu, pro který je zaslána nová prodejní cena.
                // Proměnná $response['content']['productNo'] obsahuje produktové číslo produktu
                // Proměnná $response['content']['ean'] obsahuje ean číslo produktu
                // Produktové číslo a ean lze použít k identifikaci produktu v případech, kdy ID produktu v Heurece je jiné, než ID v e-shopu nebo jej nelze použít pro jednoznačnou identifikaci produktu v e-shopu (např. u variant produktů)
                // Např.:
                // $product = new Product($response['content']['id']);

                // Kontrola, zda-li produkt s tímto ID byl nalezen
                // Proměnná $found bude mít hodnotu true, pokud byl produkt nalezen, jinak bude false
                // Např.:
                // $found = $product->id != '';  //pokud $product->id není prázné, produkt byl nalezen

                // Pokud byl produkt nalezen, pokračuj
                if ($found){

                    // Proměnná $response['content']['price'] obsahuje novou prodejní cenu s DPH.
                    // Uložte novou prodejní cenu produktu v e-shopu.
                    // Pokud váš e-shop standardně pracuje s cenami bez DPH, nazapomeňte provést přepočet na cenu bez DPH.
                    // Proměnná $success bude true, pokud se uložení ceny podaří,
                    // případně false, pokud se uložení nezdaří (např. z důvodu chyby databáze).
                    // Např.:
                    // $product->price = $response['content']['price'];
                    // $success = $product->save();

                    // VÁŠ KÓD - KONEC

                    // Odpověď pro Cenový automat, že prodejní cena byla úspěšně v e-shopu aktualizována
                    if ($success){
                        http_response_code(200);
                        echo 'Price succesfully set';
                    } else {
                        http_response_code(400);
                        echo 'Price was not set';
                    }

                } else {
                    //Chyba. Produkt se zadaným ID nebyl v e-shopu nalezen.
                    http_response_code(404);
                    echo "Product with given ID not found";
                }
            } else {
                //Chyba. ID nebylo zadáno.
                http_response_code(400);
                echo "Product ID not specified";
            }
            break;

        default:
            //Chyba. Špatně zadaná akce.
            http_response_code(400);
            echo "Invalid action";
            break;
    }
} else {
    //Chyba. Akce nebyla zadána.
    http_response_code(400);
    echo "Action not specified";
}

// KONEC SKRIPTU
